@@ -7,7 +7,7 @@ package Bio::Graphics::Browser2::TrackDumper;
 #
 ###################################################################
 
-# $Id: TrackDumper.pm 24319 2010-12-29 21:24:32Z lstein $
+# $Id: TrackDumper.pm 24428 2011-01-25 22:06:32Z lstein $
 
 # Simple track dumper, suitable for a lightweight replacement to DAS.
 # Call this way:
@@ -799,7 +799,8 @@ sub get_scan {
 	next if $l =~ /^_/;
 	next if $l =~ /:\w+/;
 	next unless    $config->fallback_setting($l => 'discoverable');
-	next if        $config->code_setting($l=>'global feature');
+# why were we skipping global feature tracks here?
+#	next if        $config->code_setting($l=>'global feature');
 	my $key      = $config->code_setting($l => 'key') || $l;
 	my $citation = $config->code_setting($l => 'citation');
 	my $data_source  = $config->code_setting($l => 'data source');
